ChartMeAddon = { } ;
function ChartMeAddon:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end
function ChartMeAddon:init()
	self.const = {}
	self.const.IN_PDMG = 0;
	self.const.IN_MDMG = 1;
	self.const.OUT_DMG = 2;
	self.const.OUT_HEAL = 3;

	self.config = {}
	self.config.debug = false
	self.config.padding = 0
	self.config.frameSizeX = 300
	self.config.frameSizeY = 300
	self.config.measure = {
		self.const.IN_PDMG,
		self.const.IN_MDMG,
		self.const.OUT_HEAL
		--,self.const.OUT_DMG
	}

	self.config.colors = {}
	self.config.colors[self.const.IN_PDMG] = {1,1,1}
	self.config.colors[self.const.IN_MDMG] = {1,1,0}
	self.config.colors[self.const.OUT_HEAL] = {0,1,0}
	self.config.colors[self.const.OUT_DMG] = {1,0,0}
	self.config.lowerAlpha = 0.2
	self.config.upperAlpha = 1

	self.recentLog = RecentLog:new()
	self.totalElapsed = 0
	self.myGuid = UnitGUID("player")
	self.barFrames = {}
	self.barTextures = {}
	
	self.frame = CreateFrame("Frame", "MyFrame", UIParent)
	self.frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self.frame:SetMovable(true)
	self.frame:EnableMouse(true)
	self.frame:SetScript("OnDragStart", self.frame.StartMoving)
	self.frame:SetScript("OnDragStop", self.frame.StopMovingOrSizing)
	self.frame:SetSize(self.config.frameSizeX, self.config.frameSizeY) 
	self.frame:SetPoint("LEFT") 
	self.frame.texture = self.frame:CreateTexture() 
	self.frame.texture:SetAllPoints(self.frame)
	self.frame.texture:SetColorTexture(.5,.5,.5,.1) 
	self.frame:RegisterForDrag("LeftButton");
	self.titleFrame = self.frame:CreateTitleRegion("ChartMe")


	self.frame.background = self.frame.texture
	self.frame.text = self.frame.text or self.frame:CreateFontString(nil,"ARTWORK","QuestFont_Shadow_Huge")
	self.frame.text:SetJustifyV("TOP")
	self.frame.text:SetAllPoints(true)
	self.frame.text:SetTextColor(0,0,0,1)
	self:initBarFrames()
	self:resetMeasures()

	local me = self

	--on event is for the combat log listener	
	self.frame:SetScript("OnEvent", function(self, event, ...)
		local timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID,
			destName, destFlags, destFlags2, spellIdOrSwingDmgAmount, spellName, spellSchool, amount = select(1, ...)
		
		me:recordEvent(
			timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, 
			destName, destFlags, destFlags2, spellIdOrSwingDmgAmount, spellName, spellSchool, amount
		);
	end)

	-- timer event - to refresh every 0.5 secs
	self.frame:SetScript("OnUpdate", function(self, elapsed)
		me:update(elapsed)
	end)

end
function ChartMeAddon:dump(o)
   if type(o) == 'table' then
      local s = '{ '
      for k,v in pairs(o) do
         if type(k) ~= 'number' then k = '"'..k..'"' end
         s = s .. '['..k..'] = ' .. self:dump(v) .. ','
      end
      return s .. '} '
   else
      return tostring(o)
   end
end
function ChartMeAddon:addMeasure(type, amount)
	self.eventData[type] = self.eventData[type] + amount
end
function ChartMeAddon:getMeasure(type)
	return self.eventData[type]
end
function ChartMeAddon:resetMeasures()
	self.eventData = {};
	for _, measure in ipairs(self.config.measure) do
		self.eventData[measure] = 0;
	end
end

function ChartMeAddon:round(num)
	return math.floor(num+0.5)
end

function ChartMeAddon:initBarFrames()
	local row = 0;
	local numCharts = #self.config.measure
	for _, measure in ipairs(self.config.measure) do
		self.barFrames[measure] = {}
		self.barTextures[measure] = {}
		local count = self.recentLog:getNumEntries()
		local x = 0;
		self.barWidth = (self.frame:GetWidth() - self.config.padding*(count+1))/count
		for i=0, count-1 do
			local minX = x+self.config.padding
			minX = self.frame:GetWidth() - minX - self.barWidth
			local maxX = minX+self.barWidth
			local minY = (row*self.frame:GetHeight()/numCharts)+self.config.padding
			local maxY = minY+1
			local f = CreateFrame("Frame", "f"..measure..i, self.frame)
			local t = f:CreateTexture("t"..measure..i, "BACKGROUND")
			local color = self.config.colors[measure];
			t:SetColorTexture(color[1],color[2],color[3], self.config.lowerAlpha)
			t:SetAllPoints()

			minX = self:round(minX)
			minY = self:round(minY)
			maxX = self:round(maxX)
			maxY = self:round(maxY)
			if self.config.debug then
				print(minX..','..minY..' '..maxX..','..maxY)
			end
			f:SetPoint("BOTTOMLEFT", minX, minY)
			f:SetSize(maxX-minX, maxY-minY)
			self.barFrames[measure][i] = f
			self.barTextures[measure][i] = t
			x = x + self.barWidth + self.config.padding
		end
		row = row + 1
	end
end
function ChartMeAddon:plotDmgGraph(entries, measure, maxDmg, count)
	--print(self.recentLog:getCount())
	for i=0, (count-1) do
		local e = entries[i]
		local dmg = e[measure]
		local frame = self.barFrames[measure][i]
		local numCharts = #self.config.measure
		local maxHeight = (self.frame:GetHeight()-self.config.padding*(numCharts+1))/numCharts
		local barHeight = self:round(maxHeight * dmg / maxDmg)
		if barHeight == 0 then
			barHeight = 1
		end
		local alpha = (dmg/maxDmg)*(self.config.upperAlpha-self.config.lowerAlpha) + self.config.lowerAlpha
		frame:SetAlpha(alpha)
		--print(dmg .. " with maxh " .. self:round(maxHeight).. " = ".. barHeight)

		frame:SetSize(self.barWidth, barHeight)
		--print(ChartMeAddon:dump(e))
	end
end

function ChartMeAddon:refreshData()
	if self.recentLog then
		local entries = self.recentLog:getAllEvents()
		
		local count = self.recentLog:getCount()
		--print ('count '..count)
		local maxDmg = 100;
		for i=0, (count-1) do
			local e = entries[i];
			for _, measure in ipairs(self.config.measure) do
				if (e[measure] > maxDmg) then
					maxDmg = e[measure]
				end
			end
		end
		
		self.frame.text:SetText(maxDmg)
		for _, measure in ipairs(self.config.measure) do
			self:plotDmgGraph(entries, measure, maxDmg, count)
		end
		--self:plotDmgGraph(1, entries, self.config.IN_MDMG, maxDmg, count)
	end
end

function ChartMeAddon:recordEvent(timestamp, type, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, destName, destFlags, destFlags2, spellIdOrSwingDmgAmount, spellName, spellSchool, amount)
	for _, measure in ipairs(self.config.measure) do
		local valToStore = 0;
		if measure == self.const.IN_PDMG then
			if destGUID == self.myGuid then
				if string.match(type, "_DAMAGE") then
					--swing dmg comes from a var shared with spellId (wtf were bliz thinking?) - add to physical dmg
					if type == 'SWING_DAMAGE' then
						valToStore = spellIdOrSwingDmgAmount
					--or if its physical add to physical dmg
					elseif spellSchool == 1 then
						valToStore = amount
					end
				end
			end
		elseif measure == self.const.IN_MDMG then
			if destGUID == chartMeAddon.myGuid then
				if string.match(type, "_DAMAGE") then
					-- if its not swing_damage and not physical then add to magical dmg
					if type ~= "SWING_DAMAGE" and spellSchool ~= 1 then
						valToStore = amount
					end
				end
			end
		end
		if measure == self.const.OUT_DMG then
			if sourceGUID == self.myGuid then
				--print "i did something"
				if string.match(type, "_DAMAGE") then
					--print ("i did some dmg")
					--swing dmg comes from a var shared with spellId (wtf were bliz thinking?) - add to physical dmg
					if type == 'SWING_DAMAGE' then
						valToStore = spellIdOrSwingDmgAmount
					--otherwise get the dmg from the "amount" field
					elseif spellSchool == 1 then
						valToStore = amount
					end
					if (self.config.debug) then
						print ("i did some dmg " .. valToStore)
					end
				end
			end
		end
		if measure == self.const.OUT_HEAL then
			if sourceGUID == self.myGuid then
				if string.match(type, "_HEAL") then
					valToStore = amount;
				end
			end
		end
		if valToStore > 0 then
			self:addMeasure(measure, valToStore)
		end
	end
end

function ChartMeAddon:update(elapsed)
	self.totalElapsed = self.totalElapsed + elapsed
  	if self.totalElapsed > 0.5 then
    	self.totalElapsed = 0;
    	self.recentLog:add(chartMeAddon.eventData);
    	if self.config.debug then
	    	print (self:getMeasure(chartMeAddon.const.IN_PDMG) .. " " .. self:getMeasure(chartMeAddon.const.IN_MDMG))
	    end
    	self:refreshData()
	--ChartMeAddon["recentLog"]:display()
    	self:resetMeasures()
  end
end

chartMeAddon = ChartMeAddon:new()
chartMeAddon:init()

